from phoebusgen.widget.widget import _Widget, _Generic
from phoebusgen.widget import properties as _p

# Displays
class Arc(_Widget, _p._Macro, _p._Angle, _p._LineWidth, _p._LineColor, _p._BackgroundColor, _p._Transparent):
    """ Arc Phoebus Widget """
    def __init__(self, name: str, x: int, y: int, width: int, height: int) -> None:
        """
        Create Arc Widget

        :param name: Widget name
        :param x: X position
        :param y: Y position
        :param width: Widget width
        :param height: Widget height
        """
        _Widget.__init__(self, 'arc', name, x, y, width, height)

class Ellipse(_Widget, _p._Macro, _p._LineWidth, _p._LineColor, _p._BackgroundColor, _p._Transparent):
    """ Ellipse Phoebus Widget """
    def __init__(self, name: str, x: int, y: int, width: int, height: int) -> None:
        """
        Create Ellipse Widget

        :param name: Widget name
        :param x: X position
        :param y: Y position
        :param width: Widget width
        :param height: Widget height
        """
        _Widget.__init__(self, 'ellipse', name, x, y, width, height)

class Label(_Widget, _p._Text, _p._Macro, _p._Font, _p._ForegroundColor, _p._BackgroundColor, _p._Transparent, _p._HorizontalAlignment,
            _p._VerticalAlignment, _p._RotationStep, _p._WrapWords, _p._AutoSize, _p._Border):
    """ Label Phoebus Widget """
    def __init__(self, name: str, text: str, x: int, y: int, width: int, height: int) -> None:
        """
        Create Label Widget

        :param name: Widget name
        :param text: Text of label
        :param x: X position
        :param y: Y position
        :param width: Widget width
        :param height: Widget height
        """
        _Widget.__init__(self, 'label', name, x, y, width, height)
        self.text(text)

class Picture(_Widget, _p._Macro, _p._File, _p._StretchToFit, _p._Rotation):
    """ Picture Phoebus Widget """
    def __init__(self, name: str, file: str, x: int, y: int, width: int, height: int) -> None:
        """
        Create Picture Widget

        :param name: Widget name
        :param file: File path to image
        :param x: X position
        :param y: Y position
        :param width: Widget width
        :param height: Widget height
        """
        _Widget.__init__(self, 'picture', name, x, y, width, height)
        self.file(file)

class Polygon(_Widget, _p._Macro, _p._LineWidth, _p._LineColor, _p._BackgroundColor, _p._Points):
    """ Polygon Phoebus Widget """
    def __init__(self, name: str, x: int, y: int, width: int, height: int) -> None:
        """
        Create Polygon Widget

        :param name: Widget name
        :param x: X position
        :param y: Y position
        :param width: Widget width
        :param height: Widget height
        """
        _Widget.__init__(self, 'polygon', name, x, y, width, height)

class Polyline(_Widget, _p._Macro, _p._LineWidth, _p._LineColor, _p._LineStyle, _p._Arrow, _p._Points):
    """ Polyline Phoebus Widget """
    def __init__(self, name: str, x: int, y: int, width: int, height: int) -> None:
        """
        Create Polyline Widget

        :param name: Widget name
        :param x: X position
        :param y: Y position
        :param width: Widget width
        :param height: Widget height
        """
        _Widget.__init__(self, 'polyline', name, x, y, width, height)

class Rectangle(_Widget, _p._Macro, _p._LineWidth, _p._LineColor, _p._BackgroundColor, _p._Transparent, _p._Corner):
    """ Rectangle Phoebus Widget """
    def __init__(self, name: str, x: int, y: int, width: int, height: int) -> None:
        """
        Create Rectangle Widget

        :param name: Widget name
        :param x: X position
        :param y: Y position
        :param width: Widget width
        :param height: Widget height
        """
        _Widget.__init__(self, 'rectangle', name, x, y, width, height)

# Monitors
class ByteMonitor(_Widget, _p._PVName, _p._StartBit, _p._NumBits, _p._ReverseBits, _p._Horizontal, _p._Square,
                  _p._OffColor, _p._OnColor, _p._ForegroundColor, _p._Font, _p._Labels, _p._AlarmBorder):
    """ ByteMonitor Phoebus Widget """
    def __init__(self, name: str, pv_name: str, x: int, y: int, width: int, height: int) -> None:
        """
        Create ByteMonitor Widget

        :param name: Widget name
        :param pv_name: Widget PV
        :param x: X position
        :param y: Y position
        :param width: Widget width
        :param height: Widget height
        """
        _Widget.__init__(self, 'byte_monitor', name, x, y, width, height)
        self.pv_name(pv_name)

class LED(_Widget, _p._PVName, _p._Bit, _p._Off, _p._On, _p._Font, _p._ForegroundColor, _p._LineColor,
          _p._Square, _p._LabelsFromPV, _p._AlarmBorder):
    """ LED Phoebus Widget """
    def __init__(self, name: str, pv_name: str, x: int, y: int, width: int, height: int) -> None:
        """
        Create LED Widget

        :param name: Widget name
        :param pv_name: Widget PV
        :param x: X position
        :param y: Y position
        :param width: Widget width
        :param height: Widget height
        """
        _Widget.__init__(self, 'led', name, x, y, width, height)
        self.pv_name(pv_name)

class LEDMultiState(_Widget, _p._PVName, _p._Font, _p._ForegroundColor, _p._LineColor, _p._Square,
                    _p._AlarmBorder, _p._States, _p._Fallback):
    """ LEDMultiState Phoebus Widget """
    def __init__(self, name: str, pv_name: str, x: int, y: int, width: int, height: int) -> None:
        """
        Create LEDMultiState Widget

        :param name: Widget name
        :param pv_name: Widget PV
        :param x: X position
        :param y: Y position
        :param width: Widget width
        :param height: Widget height
        """
        _Widget.__init__(self, 'multi_state_led', name, x, y, width, height)
        self.pv_name(pv_name)

class Meter(_Widget, _p._PVName, _p._ForegroundColor, _p._BackgroundColor, _p._Font, _p._Format,
            _p._Precision, _p._ShowValue, _p._ShowUnits, _p._ShowLimits, _p._AlarmBorder,
            _p._LimitsFromPV, _p._MinMax, _p._NeedleColor, _p._KnobColor):
    """ Meter Phoebus Widget """
    def __init__(self, name: str, pv_name: str, x: int, y: int, width: int, height: int) -> None:
        """
        Create Meter Widget

        :param name: Widget name
        :param pv_name: Widget PV
        :param x: X position
        :param y: Y position
        :param width: Widget width
        :param height: Widget height
        """
        _Widget.__init__(self, 'meter', name, x, y, width, height)
        self.pv_name(pv_name)

class ProgressBar(_Widget, _p._PVName, _p._FillColor, _p._BackgroundColor, _p._Horizontal,
                  _p._AlarmBorder, _p._LimitsFromPV, _p._MinMax):
    """ ProgressBar Phoebus Widget """
    def __init__(self, name: str, pv_name: str, x: int, y: int, width: int, height: int) -> None:
        """
        Create ProgressBar Widget

        :param name: Widget name
        :param pv_name: Widget PV
        :param x: X position
        :param y: Y position
        :param width: Widget width
        :param height: Widget height
        """
        _Widget.__init__(self, 'progressbar', name, x, y, width, height)
        self.pv_name(pv_name)

class Symbol(_Widget, _p._Symbols, _p._PVName, _p._BackgroundColor, _p._InitialIndex,
             _p._Rotation, _p._ShowIndex, _p._Transparent, _p._AlarmBorder, _p._ArrayIndex,
             _p._AutoSize, _p._Enabled, _p._PreserveRatio):
    """ Symbol Phoebus Widget """
    def __init__(self, name: str, pv_name: str, x: int, y: int, width: int, height: int) -> None:
        """
        Create Symbol Widget

        :param name: Widget name
        :param pv_name: Widget PV
        :param x: X position
        :param y: Y position
        :param width: Widget width
        :param height: Widget height
        """
        _Widget.__init__(self, 'symbol', name, x, y, width, height)
        self.pv_name(pv_name)

class Table(_Widget, _p._PVName, _p._Font, _p._ForegroundColor, _p._BackgroundColor, _p._ShowToolbar,
            _p._AlarmBorder, _p._Editable, _p._SelectRows, _p._SelectionPV, _p._Columns):
    """ Table Phoebus Widget """
    def __init__(self, name: str, pv_name: str, x: int, y: int, width: int, height: int) -> None:
        """
        Create Table Widget

        :param name: Widget name
        :param pv_name: Widget PV
        :param x: X position
        :param y: Y position
        :param width: Widget width
        :param height: Widget height
        """
        _Widget.__init__(self, 'table', name, x, y, width, height)
        self.pv_name(pv_name)

class Tank(_Widget, _p._PVName, _p._Font, _p._ForegroundColor, _p._BackgroundColor,
           _p._FillColor, _p._EmptyColor, _p._ScaleVisible, _p._AlarmBorder, _p._LimitsFromPV,
           _p._MinMax):
    """ Tank Phoebus Widget """
    def __init__(self, name: str, pv_name: str, x: int, y: int, width: int, height: int) -> None:
        """
        Create Tank Widget

        :param name: Widget name
        :param pv_name: Widget PV
        :param x: X position
        :param y: Y position
        :param width: Widget width
        :param height: Widget height:
        """
        _Widget.__init__(self, 'tank', name, x, y, width, height)
        self.pv_name(pv_name)

class TextSymbol(_Widget, _p._PVName, _p._Font, _p._ForegroundColor, _p._BackgroundColor, _p._Transparent,
                 _p._HorizontalAlignment, _p._VerticalAlignment, _p._Rotation, _p._WrapWords,
                 _p._AlarmBorder, _p._Enabled, _p._ArrayIndex, _p._Symbols):
    """ TextSymbol Phoebus Widget """
    def __init__(self, name: str, pv_name: str, x: int, y: int, width: int, height: int) -> None:
        """
        Create TextSymbol Widget

        :param name: Widget name
        :param pv_name: Widget PV
        :param x: X position
        :param y: Y position
        :param width: Widget width
        :param height: Widget height
        """
        _Widget.__init__(self, 'text-symbol', name, x, y, width, height)
        self.pv_name(pv_name)

class TextUpdate(_Widget, _p._PVName, _p._Font, _p._ForegroundColor, _p._BackgroundColor, _p._Transparent,
                 _p._Format, _p._Precision, _p._ShowUnits, _p._HorizontalAlignment, _p._VerticalAlignment, _p._WrapWords,
                 _p._RotationStep, _p._Border, _p._AlarmBorder):
    """ TextUpdate Phoebus Widget """
    def __init__(self, name: str, pv_name: str, x: int, y: int, width: int, height: int) -> None:
        """
        Create TextUpdate Widget

        :param name: Widget name
        :param pv_name: Widget PV
        :param x: X position
        :param y: Y position
        :param width: Widget width
        :param height: Widget height
        """
        _Widget.__init__(self, 'textupdate', name, x, y, width, height)
        self.pv_name(pv_name)

class Thermometer(_Widget, _p._PVName, _p._FillColor, _p._AlarmBorder, _p._LimitsFromPV, _p._MinMax):
    """ Thermometer Phoebus Widget """
    def __init__(self, name: str, pv_name: str, x: int, y: int, width: int, height: int) -> None:
        """
        Create Thermometer Widget

        :param name: Widget name
        :param pv_name: Widget PV
        :param x: X position
        :param y: Y position
        :param width: Widget width
        :param height: Widget height
        """
        _Widget.__init__(self, 'thermometer', name, x, y, width, height)
        self.pv_name(pv_name)

# Controls
class ActionButton(_Widget, _p._PVName, _p._Actions, _p._Text, _p._Font, _p._ForegroundColor, _p._BackgroundColor,
                   _p._Transparent, _p._RotationStep, _p._Enabled, _p._AlarmBorder, _p._Confirmation):
    """ ActionButton Phoebus Widget """
    def __init__(self, name: str, text: str, pv_name: str, x: int, y: int, width: int, height: int) -> None:
        """
        Create ActionButton Widget

        :param name: Widget name
        :param text: Button text
        :param pv_name: Widget PV
        :param x: X position
        :param y: Y position
        :param width: Widget width
        :param height: Widget height
        """
        _Widget.__init__(self, 'action_button', name, x, y, width, height)
        self.pv_name(pv_name)
        self.text(text)

class BooleanButton(_Widget, _p._PVName, _p._Bit, _p._OffImage, _p._OnImage, _p._ShowLED, _p._Font, _p._ForegroundColor, _p._BackgroundColor,
                    _p._LabelsFromPV, _p._AlarmBorder, _p._Enabled, _p._Mode, _p._Confirmation):
    """ BooleanButton Phoebus Widget """
    def __init__(self, name: str, pv_name: str, x: int, y: int, width: int, height: int) -> None:
        """
        Create BooleanButton Widget

        :param name: Widget name
        :param pv_name: Widget PV
        :param x: X position
        :param y: Y position
        :param width: Widget width
        :param height: Widget height
        """
        _Widget.__init__(self, 'bool_button', name, x, y, width, height)
        self.pv_name(pv_name)

class CheckBox(_Widget, _p._PVName, _p._Bit, _p._Label, _p._Font, _p._ForegroundColor, _p._AutoSize,
               _p._AlarmBorder, _p._Confirmation):
    """ CheckBox Phoebus Widget """
    def __init__(self, name: str, label: str, pv_name: str, x: int, y: int, width: int, height: int) -> None:
        """
        Create CheckBox Widget

        :param name: Widget name
        :param label: Label text
        :param pv_name: Widget PV
        :param x: X position
        :param y: Y position
        :param width: Widget width
        :param height: Widget height
        """
        _Widget.__init__(self, 'checkbox', name, x, y, width, height)
        self.pv_name(pv_name)
        self.label(label)

class ChoiceButton(_Widget, _p._PVName, _p._Font, _p._ForegroundColor, _p._BackgroundColor, _p._SelectedColor, _p._Horizontal,
                   _p._AlarmBorder, _p._Items, _p._ItemsFromPV, _p._Confirmation):
    """ ChoiceButton Phoebus Widget """
    def __init__(self, name: str, pv_name: str, x: int, y: int, width: int, height: int) -> None:
        """
        Create ChoiceButton Widget

        :param name: Widget name
        :param pv_name: Widget PV
        :param x: X position
        :param y: Y position
        :param width: Widget width
        :param height: Widget height
        """
        _Widget.__init__(self, 'choice', name, x, y, width, height)
        self.pv_name(pv_name)

class ComboBox(_Widget, _p._PVName, _p._Font, _p._ForegroundColor, _p._BackgroundColor, _p._AlarmBorder, _p._Items,
               _p._ItemsFromPV, _p._Editable, _p._Enabled, _p._Confirmation):
    """ ComboBox Phoebus Widget """
    def __init__(self, name: str, pv_name: str, x: int, y: int, width: int, height: int) -> None:
        """
        Create ComboBox Widget

        :param name: Widget name
        :param pv_name: Widget PV
        :param x: X position
        :param y: Y position
        :param width: Widget width
        :param height: Widget height
        """
        _Widget.__init__(self, 'combo', name, x, y, width, height)
        self.pv_name(pv_name)


class FileSelector(_Widget, _p._PVName, _p._FileComponent, _p._AlarmBorder, _p._Enabled):
    """ FileSelector Phoebus Widget """
    def __init__(self, name: str, pv_name: str, x: int, y: int, width: int, height: int) -> None:
        """
        Create FileSelector Widget

        :param name: Widget name
        :param pv_name: Widget PV
        :param x: X position
        :param y: Y position
        :param width: Widget width
        :param height: Widget height
        """
        _Widget.__init__(self, 'fileselector', name, x, y, width, height)
        self.pv_name(pv_name)

class RadioButton(_Widget, _p._PVName, _p._Font, _p._ForegroundColor, _p._Horizontal, _p._AlarmBorder,
                  _p._Items, _p._ItemsFromPV, _p._Enabled, _p._Confirmation):
    """ RadioButton Phoebus Widget """
    def __init__(self, name: str, pv_name: str, x: int, y: int, width: int, height: int) -> None:
        """
        Create RadioButton Widget

        :param name: Widget name
        :param pv_name: Widget PV
        :param x: X position
        :param y: Y position
        :param width: Widget width
        :param height: Widget height
        """
        _Widget.__init__(self, 'radio', name, x, y, width, height)
        self.pv_name(pv_name)

class ScaledSlider(_Widget, _p._PVName, _p._Horizontal, _p._ForegroundColor, _p._BackgroundColor, _p._Transparent, _p._Font,
                   _p._ShowScale, _p._ShowMinorTicks, _p._MajorTicksPixelDist, _p._ScaleFormat, _p._LevelsAndShow, _p._AlarmBorder,
                   _p._Increment, _p._MinMax, _p._LimitsFromPV, _p._Enabled):
    """ ScaledSlider Phoebus Widget """
    def __init__(self, name: str, pv_name: str, x: int, y: int, width: int, height: int) -> None:
        """
        Create ScaledSlider Widget

        :param name: Widget name
        :param pv_name: Widget PV
        :param x: X position
        :param y: Y position
        :param width: Widget width
        :param height: Widget height
        """
        _Widget.__init__(self, 'scaledslider', name, x, y, width, height)
        self.pv_name(pv_name)

class Scrollbar(_Widget, _p._PVName, _p._Horizontal, _p._ShowValueTip, _p._AlarmBorder, _p._MinMax,
                _p._LimitsFromPV, _p._BarLength, _p._Increment, _p._Enabled):
    """ Scrollbar Phoebus Widget """
    def __init__(self, name: str, pv_name: str, x: int, y: int, width: int, height: int) -> None:
        """
        Create Scrollbar Widget

        :param name: Widget name
        :param pv_name: Widget PV
        :param x: X position
        :param y: Y position
        :param width: Widget width
        :param height: Widget height
        """
        _Widget.__init__(self, 'scrollbar', name, x, y, width, height)
        self.pv_name(pv_name)

class SlideButton(_Widget, _p._PVName, _p._Bit, _p._Label, _p._OffColor, _p._OnColor, _p._Font, _p._ForegroundColor,
                  _p._AutoSize, _p._AlarmBorder, _p._Enabled, _p._Confirmation):
    """ SlideButton Phoebus Widget """
    def __init__(self, name: str, label: str, pv_name: str, x: int, y: int, width: int, height: int) -> None:
        """
        Create SlideButton Widget

        :param name: Widget name
        :param label: Label text
        :param pv_name: Widget PV
        :param x: X position
        :param y: Y position
        :param width: Widget width
        :param height: Widget height
        """
        _Widget.__init__(self, 'slide_button', name, x, y, width, height)
        self.pv_name(pv_name)
        self.label(label)

class Spinner(_Widget, _p._PVName, _p._Format, _p._Precision, _p._ShowUnits, _p._ForegroundColor, _p._BackgroundColor,
              _p._ButtonsOnLeft, _p._AlarmBorder, _p._MinMax, _p._LimitsFromPV, _p._Increment, _p._Enabled):
    """ Spinner Phoebus Widget """
    def __init__(self, name: str, pv_name: str, x: int, y: int, width: int, height: int) -> None:
        """
        Create Spinner Widget

        :param name: Widget name
        :param pv_name: Widget PV
        :param x: X position
        :param y: Y position
        :param width: Widget width
        :param height: Widget height
        """
        _Widget.__init__(self, 'spinner', name, x, y, width, height)
        self.pv_name(pv_name)

class TextEntry(_Widget, _p._PVName, _p._Font, _p._ForegroundColor, _p._BackgroundColor, _p._Format,
                _p._Precision, _p._ShowUnits, _p._WrapWords, _p._MultiLine, _p._AlarmBorder, _p._Enabled,
                _p._Border, _p._HorizontalAlignment, _p._VerticalAlignment):
    """ TextEntry Phoebus Widget """
    def __init__(self, name: str, pv_name: str, x: int, y: int, width: int, height: int) -> None:
        """
        Create TextEntry Widget

        :param name: Widget name
        :param pv_name: Widget PV
        :param x: X position
        :param y: Y position
        :param width: Widget width
        :param height: Widget height
        """
        _Widget.__init__(self, 'textentry', name, x, y, width, height)
        self.pv_name(pv_name)

# Plots
class StripChartTrace(_Generic, _p._Name, _p._YPV, _p._Axis, _p._TraceType, _p._Color, _p._LineWidth,
                      _p._PointType, _p._PointSize):
    def __init__(self) -> None:
        _Generic.__init__(self, 'trace')

class XYPlotTrace(StripChartTrace, _p._XPV, _p._ErrPV, _p._LineStyle):
    def __init__(self) -> None:
        super().__init__()

class StripChartYAxis(_Generic, _p._Title, _p._AutoScale, _p._LogScale, _p._MinMax, _p._ShowGrid,
                      _p._Color):
    def __init__(self) -> None:
        _Generic.__init__(self, 'y_axis')

class XYPlotYAxis(StripChartYAxis, _p._TitleFont, _p._ScaleFont, _p._OnRight):
    def __init__(self) -> None:
        super().__init__()

class ImageYAxis(_Generic, _p._Title, _p._MinMax, _p._TitleFont, _p._ScaleFont):
    def __init__(self) -> None:
        _Generic.__init__(self, 'y_axis')

class XYPlotXAxis(_Generic, _p._Title, _p._AutoScale, _p._LogScale, _p._MinMax, _p._ShowGrid, _p._TitleFont,
            _p._ScaleFont):
    def __init__(self) -> None:
        _Generic.__init__(self, 'x_axis')

class ImageXAxis(_Generic, _p._Title, _p._MinMax, _p._TitleFont, _p._ScaleFont):
    def __init__(self) -> None:
        _Generic.__init__(self, 'x_axis')

class Marker(_Generic, _p._Color, _p._PVName, _p._Interactive):
    def __init__(self) -> None:
        _Generic.__init__(self, 'marker')

class RegionOfInterest(_Generic, _p._Name, _p._Color, _p._Interactive, _p._XPV, _p._YPV, _p._WidthPV, _p._HeightPV, _p._File):
    def __init__(self) -> None:
        _Generic.__init__(self, 'roi')

class ColorBar(_Generic, _p._ScaleFont, _p._ColorBarSize):
    def __init__(self) -> None:
        _Generic.__init__(self, 'color_bar')

class ColorMapColor(_Generic):
    def __init__(self, value, red, green, blue):
        _Generic.__init__(self, 'section')
        self.root.attrib['value'] = str(value)
        self.root.attrib['red'] = str(red)
        self.root.attrib['green'] = str(green)
        self.root.attrib['blue'] = str(blue)

class DataBrowser(_Widget, _p._Macro, _p._File, _p._ShowToolbar, _p._SelectionValuePV):
    """ DataBrowser Phoebus Widget """
    def __init__(self, name: str, file: str, x: int, y: int, width: int, height: int) -> None:
        """
        Create DataBrowser Widget

        :param name: Widget name
        :param pv_name: Widget PV
        :param x: X position
        :param y: Y position
        :param width: Widget width
        :param height: Widget height
        """
        _Widget.__init__(self, 'databrowser', name, x, y, width, height)
        self.file(file)

# colormap, colorbar, xaxis, yaxis, datawidth, dataheight, interpolation, colormode, unsigneddata, autoscale,
# logscale, cursor, roi
#class Image(_Widget, _p._PVName, _p._ForegroundColor, _p._BackgroundColor, _p._ShowToolbar, _p._ColorMap, _p._ColorBar,
#            _p._XAxis, _p._YAxis, _p._AlarmBorder, _p._DataWidth, _p._Interpolation, _p._ColorMode, _p._UnsignedData,
#            _p._AutoScale, _p._LogScale, _p._MinMax, _p._Cursor, _p._RegionsOfInterest):
#    pass
class Image(_Widget, _p._PVName, _p._ForegroundColor, _p._BackgroundColor, _p._ShowToolbar,
            _p._AlarmBorder, _p._MinMax, _p._AutoScale, _p._DataHeightAndWidth, _p._UnsignedData,
            _p._LogScale, _p._Cursor, _p._Interpolation, _p._XAxis, _p._YAxis, _p._ColorMode,
            _p._RegionsOfInterest, _p._ColorBar, _p._ColorMap):
    """ Image - Incomplete Widget """
    def __init__(self, name: str, pv_name: str, x: int, y: int, width: int, height: int) -> None:
        """
        Create Image Widget

        :param name: Widget name
        :param pv_name: Widget PV
        :param x: X position
        :param y: Y position
        :param width: Widget width
        :param height: Widget height
        """
        _Widget.__init__(self, 'image', name, x, y, width, height)
        self.pv_name(pv_name)

# showgrid, labelfont, scalefont, yaxes, traces
#class StripChart(_Widget, _p._ForegroundColor, _p._BackgroundColor, _p._ShowGrid, _p._Title, _p._LabelFont, _p._ScaleFont,
#                 _p._ShowToolbar, _p._TimeRange, _p._YAxes, _p._Traces):
#    pass
class StripChart(_Widget, _p._ForegroundColor, _p._BackgroundColor, _p._ShowToolbar, _p._Title,
                 _p._TitleFont, _p._ShowLegend, _p._ShowGrid, _p._TimeRange, _p._LabelFont,
                 _p._ScaleFont, _p._YAxes, _p._Traces):
    """ StripChart - Incomplete Widget """
    def __init__(self, name: str, x: int, y: int, width: int, height: int) -> None:
        """
        Create StripChart Widget

        :param name: Widget name
        :param x: X position
        :param y: Y position
        :param width: Widget width
        :param height: Widget height
        """
        _Widget.__init__(self, 'stripchart', name, x, y, width, height)

# title, showlegend, xaxis, yaxes, traces, markers
#class XYPlot(_Widget, _p._ForegroundColor, _p._BackgroundColor, _p._GridColor, _p._Title, _p._ShowToolbar, _p._ShowLegend,
#             _p._XAxis, _p._YAxes, _p._Traces, _p._Markers):
#    pass
class XYPlot(_Widget, _p._ForegroundColor, _p._BackgroundColor, _p._ShowToolbar, _p._Title,
             _p._TitleFont, _p._GridColor, _p._XAxis, _p._YAxes, _p._Traces, _p._Markers):
    """ XYPlot - Incomplete Widget """
    def __init__(self, name: str, x: int, y: int, width: int, height: int) -> None:
        """
        Create XYPlot Widget

        :param name: Widget name
        :param x: X position
        :param y: Y position
        :param width: Widget width
        :param height: Widget height
        """
        _Widget.__init__(self, 'xyplot', name, x, y, width, height)

# Structure
class Array(_Widget, _p._PVName, _p._Macro, _p._ForegroundColor, _p._BackgroundColor, _p._AlarmBorder):
    """ Array Phoebus Widget """
    def __init__(self, name: str, pv_name: str, x: int, y: int, width: int, height: int) -> None:
        """
        Create Array Widget

        :param name: Widget name
        :param pv_name: Widget PV
        :param x: X position
        :param y: Y position
        :param width: Widget width
        :param height: Widget height
        """
        _Widget.__init__(self, 'array', name, x, y, width, height)
        self.pv_name(pv_name)

class EmbeddedDisplay(_Widget, _p._Macro, _p._File, _p._ResizeBehavior, _p._GroupName, _p._Transparent, _p._Border):
    """ EmbeddedDisplay Phoebus Widget """
    def __init__(self, name: str, file: str, x: int, y: int, width: int, height: int) -> None:
        """
        Create EmbeddedDisplay Widget

        :param name: Widget name
        :param file: File path
        :param x: X position
        :param y: Y position
        :param width: Widget width
        :param height: Widget height
        """
        _Widget.__init__(self, 'embedded', name, x, y, width, height)
        self.file(file)

class Group(_Widget, _p._Structure, _p._Macro, _p._Style, _p._Font, _p._ForegroundColor, _p._BackgroundColor, _p._Transparent,
            _p._LineColor):
    """ Group Phoebus Widget """
    def __init__(self, name: str, x: int, y: int, width: int, height: int) -> None:
        """
        Create Group Widget

        :param name: Widget name
        :param x: X position
        :param y: Y position
        :param width: Widget width
        :param height: Widget height
        """
        _Widget.__init__(self, 'group', name, x, y, width, height)

class NavigationTabs(_Widget, _p._NavTabs, _p._ActiveTab, _p._TabWidth, _p._TabSpacing, _p._TabHeight,
                     _p._SelectedColor, _p._DeselectedColor, _p._Direction, _p._Font):
    """ NavigationTabs Phoebus Widget """
    def __init__(self, name: str, x: int, y: int, width: int, height: int) -> None:
        """
        Create NavigationTabs Widget

        :param name: Widget name
        :param x: X position
        :param y: Y position
        :param width: Widget width
        :param height: Widget height
        """
        _Widget.__init__(self, 'navtabs', name, x, y, width, height)

class Tabs(_Widget, _p._Macro, _p._Tabs, _p._ActiveTab, _p._TabHeight, _p._Font, _p._BackgroundColor, _p._Direction):
    """ Tabs Phoebus Widget """
    def __init__(self, name: str, x: int, y: int, width: int, height: int) -> None:
        """
        Create Tabs Widget

        :param name: Widget name
        :param x: X position
        :param y: Y position
        :param width: Widget width
        :param height: Widget height
        """
        _Widget.__init__(self, 'tabs', name, x, y, width, height)

# Miscellaneous

class ThreeDViewer(_Widget, _p._File):
    """ ThreeDViewer Phoebus Widget """
    def __init__(self, name: str, file: str, x: int, y: int, width: int, height: int) -> None:
        """
        Create ThreeDViewer Widget

        :param name: Widget name
        :param file: File path
        :param x: X position
        :param y: Y position
        :param width: Widget width
        :param height: Widget height
        """
        _Widget.__init__(self, '3dviewer', name, x, y, width, height)
        self.file(file)

class WebBrowser(_Widget, _p._URL, _p._ShowToolbar):
    """ WebBrowser Phoebus Widget """
    def __init__(self, name: str, url: str, x: int, y: int, width: int, height: int) -> None:
        """
        Create WebBrowser Widget

        :param name: Widget name
        :param url: URL
        :param x: X position
        :param y: Y position
        :param width: Widget width
        :param height: Widget height
        """
        _Widget.__init__(self, 'webbrowser', name, x, y, width, height)
        self.url(url)



from xml.etree.ElementTree import Element, SubElement, tostring
from xml.dom import minidom
from phoebusgen._shared_property_helpers import _SharedPropertyFunctions


def prettify(elem):
    """Return a pretty-printed XML string for the Element.
        From: https://pymotw.com/3/xml.etree.ElementTree/create.html
    """
    rough_string = tostring(elem, 'utf-8')
    reparse_xml = minidom.parseString(rough_string)
    return reparse_xml.toprettyxml(indent='  ', newl='\n')

class _Generic(object):
    def __init__(self, w_type: str) -> None:
        self.root = Element(w_type)
        self._shared = _SharedPropertyFunctions(self.root)

    def find_element(self, tag: str) -> Element:
        """
        Find first XML element in widget by tag name

        :param tag: Tag name to search for
        :return: Return XML element or None if not found
        """
        elements = self.root.findall(tag)
        # check to make sure there are not more than 1 elements
        # we don't want duplicate tags
        if len(elements) > 1:
            print('Warning, more than one element of the same tag! Returning a list')
            return elements
        elif len(elements) == 0:
            return None
        else:
            return elements[0]

    def remove_element(self, tag: str) -> None:
        """
        Delete XML element in widget by tag name

        :param tag: Tag name to delete
        """
        element = self.find_element(tag)
        if element is not None:
            self.root.remove(element)

    def get_element_value(self, tag: str) -> str:
        """
        Get value of an XML element by tag name

        :param tag: Tag name to get value from
        :return: Value of XML tag
        """
        return self.find_element(tag).text

    def visible(self, visible: bool) -> None:
        """
        Change visible property for widget

        :param visible: Is widget visible?
        """
        self._shared.boolean_property(self.root, 'visible', visible)

    def __str__(self):
        return prettify(self.root)

    def __repr__(self):
        return prettify(self.root)

class _Widget(_Generic):
    """ Base Class for all Phoebus widgets """
    def __init__(self, w_type: str, name: str, x_pos: int, y_pos: int, width: int, height: int) -> None:
        """
        Base Class for all Phoebus widgets

        :param w_type: Widget type to be written into XML
        :param name: Widget name
        :param x: X position
        :param y: Y position
        :param width: Widget width
        :param height: Widget height:
        """

        super().__init__('widget')
        self.root.attrib['type'] = w_type
        self.root.attrib['version'] = '2.0.0'
        name_child = SubElement(self.root, 'name')
        name_child.text = name

        self._shared.integer_property(self.root, 'x', x_pos)
        self._shared.integer_property(self.root, 'y', y_pos)
        self._shared.integer_property(self.root, 'width', width)
        self._shared.integer_property(self.root, 'height', height)

    def version(self, version: str) -> None:
        """
        Change widget version in root widget. i.e. <widget type="textupdate" version="2.0.0">

        :param version: Version string
        """
        self.root.attrib['version'] = version

    def name(self, name: str) -> None:
        """
        Change widget name

        :param name: Widget name
        """
        self._shared.generic_property(self.root, 'name', name)

    def width(self, width: int) -> None:
        """
        Change widget width

        :param width: Width
        """
        self._shared.integer_property(self.root, 'width', width)

    def height(self, height: int) -> None:
        """
        Change widget height

        :param height: height
        """
        self._shared.integer_property(self.root, 'height', height)

    def x(self, val: int) -> None:
        """
        Change widget x position

        :param val: x
        """
        self._shared.integer_property(self.root, 'x', val)

    def y(self, val: int) -> None:
        """
        Change widget y position

        :param val: y
        """
        self._shared.integer_property(self.root, 'y', val)

    #def class_name(self, name):
    #    pass

    def rule(self, name: str, widget_property: str, pv_dict: dict,
             expression_dict: dict, value_as_expression: bool = False) -> None:
        """
        Add a rule to the widget to control a property based on some logic

        :param name: Name of the rule
        :param widget_property: Property for rule to control, i.e. name, foreground_color, etc.
        :param pv_dict: Dictionary of PVs for the rule, format - { pvName: triggerOnPV }
        :param expression_dict: Dictionary of expressions for the rules, format - { boolean expression : value }
        :param value_as_expression: Defaults to False. If True, use value as expression
        """
        root_rules = self.root.find('rules')
        if root_rules is None:
            root_rules = SubElement(self.root, 'rules')
        root_rule = SubElement(root_rules, 'rule')
        root_rule.attrib['name'] = name
        root_rule.attrib['prop_id'] = widget_property
        if value_as_expression is True:
            root_rule.attrib['out_exp'] = 'true'
        else:
            root_rule.attrib['out_exp'] = 'false'
        if expression_dict is not None:
            for expression, value in expression_dict.items():
                expression_element = SubElement(root_rule, 'exp', {'bool_exp': expression})
                if value_as_expression is True:
                    val_as_exp_element = SubElement(expression_element, 'expression')
                    val_as_exp_element.text = str(value)
                else:
                    val_element = SubElement(expression_element, 'value')
                    if 'color' in widget_property:
                        name = None
                        red = None
                        green = None
                        blue = None
                        alpha = None

                        # red, green, blue, alpha entered in a tuple (R,G,B,A)
                        # alpha optional (defaults to 255)
                        if str(type(value)) == "<class 'tuple'>":
                            red = value[0]
                            green = value[1]
                            blue = value[2]
                            alpha = '255'

                            if len(value) == 4:
                                alpha = value[3]

                        else: # predefined colors
                            name = str(value)
                        self._shared.create_color_element(val_element, name, red, green, blue, alpha, False)
                    else:
                        val_element.text = str(value)
        if pv_dict is not None:
            for pv, trigger in pv_dict.items():
                pv_element = SubElement(root_rule, 'pv_name', {'trigger': str(trigger).lower()})
                pv_element.text = pv

    def _script(self, file_name, script_contents, pv_dict, only_trigger_if_connected):
        root_scripts = self.root.find('scripts')
        if root_scripts is None:
            root_scripts = SubElement(self.root, 'scripts')
        root_script = SubElement(root_scripts, 'script')
        root_script.attrib['file'] = file_name
        if only_trigger_if_connected is False:
            root_script.attrib['check_connections'] = 'false'
        if script_contents is not None:
            # self._shared.generic_property(root_script, 'text', "<![CDATA[" + python_script + "]]>")
            # from what I can tell, Phoebus will automatically add <![CDATA[ ... ]]> after saving in editor
            self._shared.generic_property(root_script, 'text', script_contents)
        if pv_dict is not None:
            for pv, trigger in pv_dict.items():
                pv_element = SubElement(root_script, 'pv_name', {'trigger': str(trigger).lower()})
                pv_element.text = pv

    def embedded_python_script(self, python_script: str, pv_dict: dict, only_trigger_if_connected: bool = True) -> None:
        """
        Add an embedded Jython (Python) script to the widget

        :param python_script: Usually multi-line string representing the actual python code to attach to widget
        :param pv_dict: Dictionary of PVs for the script, format - { pvName: triggerOnPV }
        :param only_trigger_if_connected: Defaults to True. If False, script will run even if PVs are not connected
        """
        file_name = 'EmbeddedPy'
        self._script(file_name, python_script, pv_dict, only_trigger_if_connected)

    def embedded_javascript_script(self, js_script: str, pv_dict: dict, only_trigger_if_connected: bool = True) -> None:
        """
        Add an embedded JS script to the widget

        :param js_script: Usually multi-line string representing the actual JS code to attach to widget
        :param pv_dict: Dictionary of PVs for the script, format - { pvName: triggerOnPV }
        :param only_trigger_if_connected: Defaults to True. If False, script will run even if PVs are not connected
        """
        file_name = 'EmbeddedJs'
        self._script(file_name, js_script, pv_dict, only_trigger_if_connected)

    def external_script(self, file_name: str, pv_dict: dict, only_trigger_if_connected: bool = True) -> None:
        """
        Add an external script to the widget, either jython (.py) or javascript (.js)

        :param file_name: Path and file name of the external script to attach to widget
        :param pv_dict: Dictionary of PVs for the script, format - { pvName: triggerOnPV }
        :param only_trigger_if_connected: Defaults to True. If False, script will run even if PVs are not connected
        """
        self._script(file_name, None, pv_dict, only_trigger_if_connected)

    def tool_tip(self, tool_tip: str) -> None:
        """
        Add tool tip string to widget

        :param tool_tip: Tool tip string
        """
        child = SubElement(self.root, 'tooltip')
        child.text = tool_tip



import sys
import phoebusgen

pv_arg = sys.argv[1]

print('Received the following arg from phoebus: {}'.format(pv_arg))

my_screen = phoebusgen.screen.Screen('My Generated Screen', '/home/tford/als/hlc/phoebusgen/examples/generated.bob')

y = 0
for i in range(int(pv_arg)):
    idx = str(i)
    label = phoebusgen.widget.Label('label{}'.format(idx), 'Label {}'.format(idx), 0, y, 100, 20)
    my_screen.add_widget(label)
    y += 30

my_screen.write_screen()
